import {
  onBtnRightClick,
  onBtnLeftClick,
  getBlocksSizes,
  showArrows,
  scaleImage,
  dropAppend,
  dragAppend,
  getRandomPositionToCard,
  checkingAnswerReset,
  checkingAnswerPositive,
  shuffleCards,
  removeActiveCardClass,
  renderCheckPanel,
  getCheckPanelElements,
  renderSliderWithButtonsMarkup,
  createTemplateBaseMarkup,
  togglePointerEventElement,
} from '../../../../_common_files/common_files/common_scripts.js'
;(() => {
  // это уникальный id для данного задания, который был присвоен в html

  const taskId = 'dnd_sorterGame_task-1'

  //картинка на общий фон
  const backgroundImage = 'Images_1/tablecloth.jpg'
  // const backgroundImage = "";
  // массив входящих картинок для поля куда переносятся картинки
  // в поле answerTag указывается ключевое слово для сопоставления поля для сбрасывания и картинки для сбрасывания
  // ВАЖНО!по значению "rectangle" в answerTag идет присвоение класса для параллелепипеда(прямоугольника) с отдельными размерами под него
  // imgSrc - бекграунд-картинка для поля, куда перетаскиваем элементы

  const dropCards = [
    {
      id: 1,
      name: '',
      imgSrc: 'Images_1/squareForDrop_new.png',
      answerTag: 'square',
    },
    {
      id: 2,
      name: '',
      imgSrc: 'Images_1/circleForDrop_new.png',
      answerTag: 'circle',
    },
    {
      id: 3,
      name: '',
      imgSrc: 'Images_1/triangleForDrop_new.png',
      answerTag: 'triangle',
    },
    {
      id: 4,
      name: '',
      imgSrc: 'Images_1/parallForDrop_new.png',
      answerTag: 'rectangle',
    },
  ]
  //массив входящих картинок для поля откуда картинки будут переноситься(любое количество)
  //в поле answerTag указывается ключевое слово для сопоставления поля для сбрасывания и картинки для сбрасывания
  // ВАЖНО!по значению "rectangle" в answerTag идет присвоение класса для параллелепипеда(прямоугольника) с отдельными размерами под него
  // imgSrc - картинка

  const dragCards = [
    {
      id: 1,
      name: '',
      imgSrc: 'Images_1/ball_blue.png',
      answerTag: 'circle',
    },
    {
      id: 2,
      name: '',
      imgSrc: 'Images_1/ball_green.png',
      answerTag: 'circle',
    },
    {
      id: 3,
      name: '',
      imgSrc: 'Images_1/ball_red.png',
      answerTag: 'circle',
    },
    {
      id: 4,
      name: '',
      imgSrc: 'Images_1/ball_yellow.png',
      answerTag: 'circle',
    },
    {
      id: 5,
      name: '',
      imgSrc: 'Images_1/bar_blue.png',
      answerTag: 'rectangle',
    },
    {
      id: 6,
      name: '',
      imgSrc: 'Images_1/bar_green.png',
      answerTag: 'rectangle',
    },
    {
      id: 7,
      name: '',
      imgSrc: 'Images_1/bar_red.png',
      answerTag: 'rectangle',
    },
    {
      id: 8,
      name: '',
      imgSrc: 'Images_1/cube_blue.png',
      answerTag: 'square',
    },
    {
      id: 9,
      name: '',
      imgSrc: 'Images_1/cube_green.png',
      answerTag: 'square',
    },
    {
      id: 10,
      name: '',
      imgSrc: 'Images_1/cube_red.png',
      answerTag: 'square',
    },
    {
      id: 11,
      name: '',
      imgSrc: 'Images_1/cube_yellow.png',
      answerTag: 'square',
    },
    {
      id: 12,
      name: '',
      imgSrc: 'Images_1/parallelepiped_yellow.png',
      answerTag: 'rectangle',
    },
    {
      id: 13,
      name: '',
      imgSrc: 'Images_1/roof_blue.png',
      answerTag: 'triangle',
    },
    {
      id: 14,
      name: '',
      imgSrc: 'Images_1/roof_green.png',
      answerTag: 'triangle',
    },
    {
      id: 15,
      name: '',
      imgSrc: 'Images_1/roof_red.png',
      answerTag: 'triangle',
    },
    {
      id: 16,
      name: '',
      imgSrc: 'Images_1/roof_yellow.png',
      answerTag: 'triangle',
    },
  ]
  // сама функция, которая запускается, здесь ничего менять не нужно

  renderDndSorterGame(dropCards, dragCards, taskId, backgroundImage)
})()

function renderDndSorterGame(
  dropCards,
  dragCards,
  taskId,

  backgroundImage
) {
  let draggingItem
  let elemBelow

  const taskWrapper = document.getElementById(`${taskId}`)
  createTemplateBaseMarkup(taskWrapper, createBaseMarkup)

  const dropBox = taskWrapper.querySelector('.dnd_sorterGame_dropPlaceWrapper')
  const dragBoxWrapper = taskWrapper.querySelector(
    '.dnd_sorterGame_dragPlaceWrapper'
  )
  // const leftBtn = taskWrapper.querySelector('.arrowButton_left_event')
  // const rightBtn = taskWrapper.querySelector('.arrowButton_right_event')
  const {
    leftBtn,
    rightBtn,
    sliderBox: dragBox,
  } = renderSliderWithButtonsMarkup(
    dragBoxWrapper,
    'dnd_sorterGame_sliderContent'
  )

  if (backgroundImage) {
    dropBox.style.backgroundImage = `url(${backgroundImage})`
    dropBox.style.cursor = 'zoom-in'
  }

  dropBox.insertAdjacentHTML(
    'beforeend',
    createDropPictureCardsMarkup(dropCards)
  )
  dragBox.insertAdjacentHTML(
    'beforeend',
    createDragPictureCardsMarkup(shuffleCards([...dragCards]))
  )

  const dropPlaces = taskWrapper.querySelectorAll(
    '.dnd_sorterGame_dropPlace_imageBox'
  )

  renderCheckPanel(taskWrapper, false)
  const { btnReset, controlsBox, infoBox } = getCheckPanelElements(taskWrapper)

  let sliderSet = {
    sliderItemWidth: 0,
    sliderSize: 0,
    sliderWrapperSize: dragBox.offsetParent.clientWidth,
    sliderShift: 0,
  }

  getBlocksSizes(sliderSet, dragBox)
  showArrows(sliderSet, leftBtn, rightBtn)

  taskWrapper.addEventListener('pointerdown', mouseDown)
  btnReset.addEventListener('click', onBtnResetClick)

  leftBtn.addEventListener('click', LeftClick)
  rightBtn.addEventListener('click', RightClick)
  dropBox.addEventListener('pointerdown', onDropBoxClick)

  function onDropBoxClick(event) {
    if (event.button !== 0) return
    if (
      event.target.classList.contains('dnd_sorterGame_dropPlace_imageBox') ||
      (event.target.classList.contains('dnd_sorterGame_dropPlaceWrapper') &&
        dropBox.style.backgroundImage)
    )
      scaleImage(event.target)
  }

  function LeftClick() {
    onBtnLeftClick(sliderSet, dragBox, leftBtn, rightBtn)
  }

  function RightClick() {
    onBtnRightClick(sliderSet, dragBox, leftBtn, rightBtn)
  }

  function onBtnResetClick() {
    ;[...dropPlaces].forEach(item => {
      ;[...item.children].forEach(el => {
        getRandomPositionToCard(el)
        removeActiveCardClass(el)
        el.style.position = 'relative'
        dragBox.appendChild(el)
      })
    })

    getBlocksSizes(sliderSet, dragBox)
    showArrows(sliderSet, leftBtn, rightBtn)

    checkingAnswerReset(controlsBox, infoBox)
    draggingItem = null
    if (taskWrapper.firstElementChild.classList.contains('noEventElement')) {
      togglePointerEventElement(taskWrapper.firstElementChild)
    }
  }

  function mouseDown(event) {
    if (event.button !== 0) return

    if (event.target.classList.contains('dnd_sorterGame_dragPicture_box')) {
      draggingItem = event.target
    } else return

    const findIdx = [...dragBox.children].findIndex(el => el === draggingItem)

    draggingItem.style.cursor = 'grabbing'
    draggingItem.style.touchAction = 'none' //ОБЯЗАТЕЛЬНОЕ УСЛОВИЕ(МОЖНО УБРАТЬ И ПРОПИСАТЬ В СТИЛЬ САМОМУ ОБЪЕКТУ)

    let shiftX = event.clientX - draggingItem.getBoundingClientRect().left
    let shiftY = event.clientY - draggingItem.getBoundingClientRect().top
    // ЛИМИТЫ КООРДИНАТ ОГРАНИЧИВАЮЩИЕ ВЫЛЕТ ПЕРЕТАСКИВАЕМОГО ЭЛЕМЕНТА ЗА БЛОК
    //  (ПО УМОЛЧАНИЮ interact_zadanie - РОДИТЕЛЬ ВАШЕГО БЛОКА)
    let limits = {
      top: taskWrapper.offsetTop,
      right: taskWrapper.offsetWidth + taskWrapper.offsetLeft,
      bottom: taskWrapper.offsetHeight + taskWrapper.offsetTop,
      left: taskWrapper.offsetLeft,
    }

    function moveAt(pageX, pageY) {
      draggingItem.style.left = pageX - shiftX + 'px'
      draggingItem.style.top = pageY - shiftY + 'px'
    }

    let clickWithoutMove = true

    function onMouseMove(event) {
      if (clickWithoutMove) {
        draggingItem.style.position = 'absolute'
        draggingItem.style.zIndex = 1000
        taskWrapper.appendChild(draggingItem)
      }

      let newLocation = {
        x: limits.left,
        y: limits.top,
      }
      if (event.pageX > limits.right) {
        newLocation.x = limits.right
      } else if (event.pageX > limits.left) {
        newLocation.x = event.pageX
      }
      if (event.pageY > limits.bottom) {
        newLocation.y = limits.bottom
      } else if (event.pageY > limits.top) {
        newLocation.y = event.pageY
      }

      clickWithoutMove = false
      moveAt(newLocation.x, newLocation.y)

      if (!event.composedPath().includes(draggingItem)) {
        window.addEventListener('pointerup', moveOut)
      }
      if (event.composedPath().includes(draggingItem)) {
        window.removeEventListener('pointerup', moveOut)
      }

      draggingItem.style.visibility = 'hidden'
      elemBelow = document.elementFromPoint(event.clientX, event.clientY)
      draggingItem.style.visibility = 'visible'

      if (!elemBelow) return
    }
    document.addEventListener('pointermove', onMouseMove)

    // КОГДА ВО ВРЕМЯ ПЕРЕТАСКИВАНИЯ КУРСОР ВЫНЕСЛИ ЗА ПРЕДЕЛЫ ОКНА БРАУЗЕРА И ОТПУСТИЛИ ЗАХВАТ ЭЛЕМЕНТА
    function moveOut(e) {
      dragAppend(dragBox, draggingItem, findIdx)

      window.removeEventListener('pointerup', moveOut)
      document.removeEventListener('pointermove', onMouseMove)
      draggingItem.removeEventListener('pointerup', onpointerup)
    }
    draggingItem.addEventListener('pointerup', onpointerup)

    function onpointerup(event) {
      document.removeEventListener('pointermove', onMouseMove)
      draggingItem.style.cursor = 'grab'
      if (!clickWithoutMove) {
        const elemBelowClosest = elemBelow.closest(
          '.dnd_sorterGame_dropPlace_imageBox'
        )
        if (
          elemBelowClosest &&
          elemBelowClosest.attributes.getNamedItem('drop-data').value ===
            draggingItem.attributes.getNamedItem('drag-data').value
        ) {
          dropAppend(elemBelowClosest, draggingItem)
          ;[...elemBelowClosest.children].forEach(el => {
            el.style.position = 'absolute'
          })

          if (dragBox.children.length === 0) {
            checkingAnswerPositive(controlsBox, infoBox)
            if (
              !taskWrapper.firstElementChild.classList.contains(
                'noEventElement'
              )
            ) {
              togglePointerEventElement(taskWrapper.firstElementChild)
            }
          }
          // }
          // if (
          //   elemBelow.classList.contains('dnd_sorterGame_dropPlace_imageBox') &&
          //   elemBelow.attributes.getNamedItem('drop-data').value ===
          //     draggingItem.attributes.getNamedItem('drag-data').value
          // ) {
          //   dropAppend(elemBelow, draggingItem)
          //   ;[...elemBelow.children].forEach(el => {
          //     el.style.position = 'absolute'
          //   })

          //   if (dragBox.children.length === 0) {
          //     checkingAnswerPositive(controlsBox, infoBox)
          //   }
          // } else if (
          //   elemBelow.classList.contains('dnd_sorterGame_dragPicture_box') &&
          //   !elemBelow.parentElement.classList.contains(
          //     'dnd_sorterGame_slider_box'
          //   ) &&
          //   elemBelow.parentElement.attributes.getNamedItem('drop-data').value ===
          //     draggingItem.attributes.getNamedItem('drag-data').value
          // ) {
          //   dropAppend(elemBelow.parentElement, draggingItem)
          //   ;[...elemBelow.parentElement.children].forEach(el => {
          //     el.style.position = 'absolute'
          //   })

          //   if (dragBox.children.length === 0) {
          //     checkingAnswerPositive(controlsBox, infoBox)
          //   }
        } else {
          dragAppend(dragBox, draggingItem, findIdx)
        }
        sliderSet.sliderSize = dragBox.scrollWidth
        showArrows(sliderSet, leftBtn, rightBtn)
        elemBelow = null
      } else if (
        event.target.classList.contains('dnd_sorterGame_dragPicture_box')
      ) {
        scaleImage(event.target)
      }

      draggingItem.removeEventListener('pointerup', onpointerup)
    }
  }

  function createDropPictureCardsMarkup(pictures) {
    return pictures
      .map(picture => {
        const boxSizesClass =
          picture.answerTag === 'rectangle'
            ? 'dnd_sorterGame_rectangleSizes'
            : 'dnd_sorterGame_boxSizes'

        return `<div class="dnd_sorterGame_dropPlace ${boxSizesClass}">

            <div drop-data="${picture.answerTag}" class="dnd_sorterGame_dropPlace_imageBox" style="background-image: url(${picture.imgSrc})"></div>
              </div>`
      })
      .join('')
  }
  function createDragPictureCardsMarkup(pictures) {
    return pictures
      .map(picture => {
        const boxSizesClass =
          picture.answerTag === 'rectangle'
            ? 'dnd_sorterGame_rectangleSizes'
            : 'dnd_sorterGame_boxSizes'

        return `<div
                class="dnd_sorterGame_dragPicture_box oneMultiChoice_border ${boxSizesClass}"
                draggable="false"
                drag-data="${picture.answerTag}"
                style="background-image: url(${picture.imgSrc})"
              >


              </div>`
      })
      .join('')
  }

  function createBaseMarkup() {
    return `
  <div class="dnd_sorterGame_comparePicturesWrapper">
        <div class="dnd_sorterGame_dropPlaceWrapper"></div>
        <div class="dnd_sorterGame_dragPlaceWrapper"></div>
      </div>
      `
  }
}
